

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <util/delay.h>
#include "lcd.h"
#include "clock.h"
#include "usart.h"
#include "config.h"
#include "ds1821.h"


#define FALSE 0
#define TRUE 1


#define TI_DCF  10
#define TI_TEMP 10			                		//Tasktimer der LCD Testroutine
#define TI_MOND 1


volatile uint8_t wartezeit;
volatile uint8_t DCF_TIMER;
volatile uint8_t TEMP_TIMER;
volatile uint8_t MOND_TIMER;


void DCF_ANZEIGE (void);							//Deklaration der LCD Testfunktion



ISR(TIMER0_OVF_vect) 								// Interrupt alle Millisekunde
{
	static uint16_t sekunden = 1000;

	TCNT0=131;										//Preloader um 1ms zu erreichen

	
	
	
	//Millisekunden Timer---------------

	//Tasktimer dekrementieren bis sie null sind
	
	if (DCF_TIMER) DCF_TIMER--;	
	if (TEMP_TIMER) TEMP_TIMER--;
	if (TEMP_TIMER) TEMP_TIMER--;



	
	if (--sekunden == 0)				//zuerst dekrementieren, dann schauen ob 0
	{
		sekunden = 1000;				//1000 ms laden um Sekundentakt zu erreichen
		if (wartezeit) wartezeit--;		//Wartezeit dekrementieren wenn nicht null
		if (MOND_TIMER) MOND_TIMER--;
	
	}
		

}

int main(void)
{
	//---- Initialisierung Port IO -------------------------------

	DDRC = 0x80;     					// Pin 8 von Port C auf Eingang, Rest auf Ausgang fr LCD

	
	
	
	TCCR0= (1<<CS01) | (1<<CS00); 		// Mode: Normal, Timer 0 Takt: CLK / 64			
	TCNT0 = 131;				  		// Count Register laden 
	TIMSK = (1<<TOIE0)|(1<<OCIE2);		// Timer 0 Output Compare Interrupt freigeben		

	
	DDRB = 0xFF;		// Ganzen Port B auf Ausgang schalten 
	PORTB = 0x00;		// alle LEDs aus

	DDRA = 0;			// Port A auf Eingang schalten (nicht unbedingt notwendig, ist so Start-Zustand)

	
	//----Initialisierung fr Timer 2 (PWM Steuerung)---------------------------------------------------
	TCCR2 = (1 << WGM21) | (1 << CS20); 
	
	OCR2 = 255;				

	
	Start_Clock();
	

	lcd_init(LCD_DISP_ON);		  		//Initialisierung LCD		
	lcd_clrscr();		  	

       	
	
	sei();						  				// globale Interruptfreigabe



	while (1)
	
		{
			if (DCF_TIMER ==0)					//wenn Tasktimer fr LCD Test gleich null
				{
					DCF_TIMER = TI_DCF;			//Tasktimer wieder laden	
					DCF_ANZEIGE();				//LCD Test ausfhren
				}

			if (TEMP_TIMER ==0)					//Thermometer Timer = 0
				{	
					TEMP_TIMER = TI_TEMP;		//Thermometer Timer wieder laden	
					THERMOMETER();				//Thermometer Funktion ausfhren
				}

			if (MOND_TIMER == 0)
				{
					MOND_TIMER = TI_MOND;
					MOND();
				}
	
	}

	return 0;
}



	
//-----------------------------------LCD Testroutine----------------------------------------------------------			
		
		

		void DCF_ANZEIGE (void)

		{
				
			char buffer[10];
			char temp_buffer[10];
			char zeichen[10];

			lcd_gotoxy(0,0);
			lcd_puts("Time:");
			itoa(hh,buffer,10);
			lcd_gotoxy(6,0);
			if (hh<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(8,0);
			lcd_puts(":");

			itoa(mm,buffer,10);
			if (mm<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(11,0);
			lcd_puts(":");

			itoa(ss,buffer,10);
			if (ss<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(12,0);
		
				
			itoa (flags.dcf_sync, buffer, 10);
			if (flags.dcf_sync)
				{
					lcd_gotoxy(4,3);
					lcd_puts("OK    ");
				}
			else
				{
					lcd_gotoxy(4,3);
					lcd_puts("NO SIG");
				}

			
			lcd_gotoxy(0,1);
			lcd_puts("Date:");
			itoa (day, buffer, 10);
			lcd_gotoxy(6,1);
			if (day<10)
			lcd_puts ("0");
			lcd_puts(buffer);
			lcd_gotoxy(8,1);
			lcd_puts(".");

			itoa (mon, buffer, 10);
			if (mon<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(11,1);
			lcd_puts(".");

			itoa(year, buffer, 10);
			if (year<10)
			lcd_puts("0");
			lcd_puts(buffer);
			lcd_gotoxy(15,3);
			
		
			itoa (flags.dcf_rx, buffer, 10);
			lcd_puts (buffer);


			
			lcd_gotoxy(0,2);
			lcd_puts("W.Temp:");
			lcd_gotoxy(14,2);
			lcd_puts("\xDF" "C");				
			lcd_gotoxy(0,3);
			lcd_puts("DCF:");

			lcd_gotoxy( 9,2 );
		   	sprintf( s, "%3d.%d", t, r );
		   	lcd_puts( s );

			

			
		}

		


			
